package com.chinac.sdk.demo.remote;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.chinac.remotesdk.CameraView;
import com.chinac.remotesdk.RemoteSdk;

public class CameraActivity extends Activity {
    final static String TAG = "CameraActivity";
    CameraView cameraView;
    private static final String[] NEEDED_PERMISSIONS = new String[]{
            Manifest.permission.CAMERA,//相机
            Manifest.permission.RECORD_AUDIO, //话筒
    };
    private static final int ACTION_REQUEST_PERMISSIONS = 0x001;
    RemoteSdk remoteSdk;
    String url;

    private ViewGroup overlay;
    private ProgressBar progressBar;
    private ViewGroup connectLayout;
    private TextView errMsg;
    private TextView btnReconnect;
    private TextView switchCamera;
    private boolean cameraType = false;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_camera);
        cameraView = findViewById(R.id.cameraView);
        overlay = findViewById(R.id.overlay);
        progressBar = findViewById(R.id.loading);
        connectLayout = findViewById(R.id.connectLayout);
        errMsg = findViewById(R.id.errMsg);
        btnReconnect = findViewById(R.id.btnReconnect);
        btnReconnect.setOnClickListener(v->{
            remoteSdk.connectSignal(this.url);
        });
        switchCamera = findViewById(R.id.switch_camera);
        switchCamera.setOnClickListener(v->{
            cameraType = !cameraType;
            remoteSdk.switchCamera(cameraType);
            cameraView.setMirror(cameraType);// 前置摄像头翻两转画面
        });

        this.url = getIntent().getStringExtra("url");
        if (TextUtils.isEmpty(url)) {
            Toast.makeText(this, "url is null", Toast.LENGTH_SHORT).show();
            return;
        }
        remoteSdk = new RemoteSdk(getApplicationContext());
        remoteSdk.init();
        remoteSdk.setCameraView(cameraView);
        remoteSdk.setRemoteSdkListener(new RemoteSdk.RemoteSdkListener() {
            @Override
            public void onSignalConnStatus(RemoteSdk.ConnStatus status, String code, String msg) {
                Log.d(TAG,"onSignalConnStatus " + " status " + status + " code: " + code + " msg " + msg);
                switch (status) {
                    case CONNECTING:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.VISIBLE);
                        connectLayout.setVisibility(View.GONE);
                        break;
                    case CONNECTED:
                        progressBar.setVisibility(View.GONE);
                        connectLayout.setVisibility(View.GONE);
                        overlay.setVisibility(View.GONE);
                        // 默认用哪个摄像头 true 前置摄像头  false 后置摄像头， 实际会受云手机内调用摄像头的APP控制
                        remoteSdk.connectCamera(true);
                        break;
                    case DISCONNECTED:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.GONE);
                        errMsg.setText("连接断开: " + code + " " + getErrMsg(code));
                        connectLayout.setVisibility(View.VISIBLE);
                        remoteSdk.disconnectCamera();
                        break;
                }
            }

            @Override
            public void onVideoConnStatusChange(RemoteSdk.ConnStatus status, String code, String msg) {

            }

            @Override
            public void onVideoOrientationChange(RemoteSdk.OrientationEnum orientation, int width, int height) {

            }

            @Override
            public void onCameraAudioConnStatusChange(RemoteSdk.ConnStatus status, String code, String msg) {
                switch (status) {
                    case CONNECTING:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.VISIBLE);
                        connectLayout.setVisibility(View.GONE);
                        break;
                    case CONNECTED:
                        progressBar.setVisibility(View.GONE);
                        connectLayout.setVisibility(View.GONE);
                        overlay.setVisibility(View.GONE);
                        break;
                    case DISCONNECTED:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.GONE);
                        errMsg.setText("连接断开: " + code + " " + getErrMsg(code));
                        connectLayout.setVisibility(View.VISIBLE);
                        break;
                }
            }

            @Override
            public void onCopy(String text) {

            }
        });
        if (checkPermissions(NEEDED_PERMISSIONS)) {
            Log.d(TAG, "conn signal");
            remoteSdk.connectSignal(url);
        } else {
            ActivityCompat.requestPermissions(this, NEEDED_PERMISSIONS,
                    ACTION_REQUEST_PERMISSIONS);
            // wait for permission
            Log.d(TAG, "wait for permission");
        }
    }

    public String getErrMsg(String code) {
        if (RemoteSdk.ErrCode.INVALID_TOKEN.equals(code)) {
            return "会话过期";
        } else if (RemoteSdk.ErrCode.CONNECT_TOO_OFTEN.equals(code)){
            return "连接太频繁，5秒后再试";
        } else if (RemoteSdk.ErrCode.INVALID_TOKEN.equals(code)){
            return "连接太频繁，5秒后再试";
        } else if (RemoteSdk.ErrCode.TIMEOUT.equals(code)){
            return "连接超时";
        } else if (RemoteSdk.ErrCode.VIDEO_DISCONNECTED.equals(code)){
            return "视频流断开";
        } else if (RemoteSdk.ErrCode.SINGNAL_DISCONNECTED.equals(code)){
            return "Websocket断开";
        } else if (RemoteSdk.ErrCode.INVALID_URL.equals(code)){
            return "URL格式不合法";
        }
        return "未知错误";
    }


    /** 权限检查
     * @param neededPermissions 需要的权限
     * @return 是否全部被允许
     */
    protected boolean checkPermissions(String[] neededPermissions) {
        if (neededPermissions == null || neededPermissions.length == 0) {
            return true;
        }
        boolean allGranted = true;
        for (String neededPermission : neededPermissions) {
            allGranted &= ContextCompat.checkSelfPermission(this, neededPermission) == PackageManager.PERMISSION_GRANTED;
        }
        return allGranted;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == ACTION_REQUEST_PERMISSIONS) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED && grantResults[1] == PackageManager.PERMISSION_GRANTED) {
                remoteSdk.connectSignal(this.url);
            } else {
                Toast.makeText(this, "Not permission granted", Toast.LENGTH_SHORT).show();
                finish();
            }
        }
    }
    public static void start(Context context, String url) {
        Intent starter = new Intent(context, CameraActivity.class);
        starter.putExtra("url", url);
        context.startActivity(starter);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (remoteSdk != null) {
            remoteSdk.disconnectSignal();
            remoteSdk.deInit();
        }
        if (cameraView != null) {
            cameraView.release();
        }
    }
}