package com.chinac.sdk.demo.remote;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.chinac.sdk.demo.remote.utils.SpUtils;


public class MainActivity extends Activity {
    final String TAG = "MainActivity";
    Button connBtn;
    Button connCameraBtn;
    EditText mobileUrlEt;
    EditText cameraUrlEt;
    private static final int ACTION_REQUEST_PERMISSIONS = 0x001;
    /**
     * 所需的所有权限信息
     */
    private static final String[] NEEDED_PERMISSIONS = new String[]{
            Manifest.permission.WRITE_EXTERNAL_STORAGE,//写入外部存储
            Manifest.permission.READ_EXTERNAL_STORAGE,//读取外部存储
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.activity_main);
        connBtn = findViewById(R.id.conn_btn);
        connCameraBtn = findViewById(R.id.conn_camera_btn);
        mobileUrlEt = findViewById(R.id.mobile_url_et);
        mobileUrlEt.setText(SpUtils.inst(getApplicationContext()).getMobileUrl());
        cameraUrlEt = findViewById(R.id.camera_url_et);
        cameraUrlEt.setText(SpUtils.inst(getApplicationContext()).getCameraUrl());
        connBtn.setOnClickListener(this::onClick);
        connCameraBtn.setOnClickListener(this::onClick);

        if (!checkPermissions(NEEDED_PERMISSIONS)) {
            ActivityCompat.requestPermissions(this, NEEDED_PERMISSIONS,
                    ACTION_REQUEST_PERMISSIONS);
        }
    }

     /** 权限检查
     * @param neededPermissions 需要的权限
     * @return 是否全部被允许
     */
    protected boolean checkPermissions(String[] neededPermissions) {
        if (neededPermissions == null || neededPermissions.length == 0) {
            return true;
        }
        boolean allGranted = true;
        for (String neededPermission : neededPermissions) {
            allGranted &= ContextCompat.checkSelfPermission(this, neededPermission) == PackageManager.PERMISSION_GRANTED;
        }
        return allGranted;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    private void onClick(View v) {

        switch (v.getId()) {
            case R.id.conn_btn:
                String url = mobileUrlEt.getText().toString();
                SpUtils.inst(getApplicationContext()).setMobileUrl(url);
                RemoteActivity.start(this,url);
                break;
            case R.id.conn_camera_btn:
                String cameraUrl = cameraUrlEt.getText().toString();
                SpUtils.inst(getApplicationContext()).setCameraUrl(cameraUrl);
                CameraActivity.start(this, cameraUrl);
                break;
        }
    }

}