package com.chinac.sdk.demo.remote;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.chinac.remotesdk.RemoteSdk;
import com.chinac.remotesdk.RemoteView;
import com.chinac.remotesdk.bean.ResponseCode;
import com.chinac.remotesdk.request.IResponseCallback;
import com.chinac.sdk.demo.remote.utils.ClipboardUtils;
import com.chinac.sdk.demo.remote.utils.SpUtils;
import com.chinac.sdk.demo.remote.widget.BottomMenuDialog;
import com.chinac.sdk.demo.remote.widget.FloatBall;
import com.chinac.sdk.demo.remote.widget.InputDialog;
import com.chinac.sdk.demo.remote.widget.ResolutionDialog;

public class RemoteActivity extends Activity implements View.OnClickListener {
    private static final String TAG = "RemoteActivity";
    private  static final String KEY_URL = "url";
    private static final String[] NEEDED_PERMISSIONS = new String[]{
            Manifest.permission.CAMERA,//相机
            Manifest.permission.RECORD_AUDIO, //话筒
    };
    private static final int ACTION_REQUEST_PERMISSIONS = 0x001;
    private Context context;
    private RemoteView remoteView;
    private ImageView btnBack;
    private ImageView btnHome;
    private ImageView btnTask;
    private ImageView btnVoiceUp;
    private ImageView btnVoiceDown;
    private TextView btnResolution;
    private ImageView btnMore;
    private String url;
    private FloatBall floatBall;


    private ViewGroup overlay;
    private ProgressBar progressBar;
    private ViewGroup connectLayout;
    private TextView errMsg;
    private TextView btnReconnect;

    private View bottomBar;
    private BottomMenuDialog bottomMenuDialog;
    private InputDialog inputDialog;
    private ResolutionDialog resolutionDialog;

    RemoteSdk remoteSdk;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_remote);
        context = this;
        remoteSdk = new RemoteSdk(getApplicationContext());

        remoteSdk.init();
        remoteView = findViewById(R.id.remoteView);
        overlay = findViewById(R.id.overlay);
        progressBar = findViewById(R.id.loading);
        connectLayout = findViewById(R.id.connectLayout);
        errMsg = findViewById(R.id.errMsg);
        btnReconnect = findViewById(R.id.btnReconnect);
        btnReconnect.setOnClickListener(v->{
            remoteSdk.connectSignal(this.url);
        });
        bottomBar = findViewById(R.id.bottomButton);
        initBottomMenu();
        initInputDialog();
        initResolutionDialog();
        btnBack = findViewById(R.id.btnBack);
        btnBack.setOnClickListener(this);
        btnHome = findViewById(R.id.btnHome);
        btnHome.setOnClickListener(this);
        btnTask = findViewById(R.id.btnTask);
        btnTask.setOnClickListener(this);
        btnVoiceUp = findViewById(R.id.btnVoiceUp);
        btnVoiceUp.setOnClickListener(this);
        btnVoiceDown = findViewById(R.id.btnVoiceDown);
        btnVoiceDown.setOnClickListener(this);
        btnResolution = findViewById(R.id.btnResolution);
        btnResolution.setOnClickListener(this);
        btnMore = findViewById(R.id.btnMore);
        btnMore.setOnClickListener(this);
//        btnCamera = findViewById(R.id.btn_camera);
//        btnCamera.setOnClickListener(this);
        setCameraBtn(SpUtils.inst(getApplicationContext()).getCameraEnable());
        floatBall = findViewById(R.id.floatBall);
        floatBall.setOnItemClickListener(new FloatBall.OnItemClickListener() {
            @Override
            public void onItemClick(int index, int resId) {
                Log.d(TAG,  " onItemCLick " + index + " resId: " + resId);
                switch (resId) {
                    case R.drawable.btn_home:
                        remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_HOME);
                        break;
                    case R.drawable.btn_back:
                        remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_BACK);
                        break;
                    case R.drawable.btn_task:
                        remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_APP_SWITCH);
                        break;
                    case R.drawable.btn_voice_up:
                        remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_VOLUME_UP);
                        break;
                    case R.drawable.btn_voice_down:
                        remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_VOLUME_DOWN);
                        break;
                    case R.drawable.btn_more:
                        floatBall.close();
                        bottomMenuDialog.show();
                        break;
                }
            }
        });
        this.url = getIntent().getStringExtra(KEY_URL);
        Log.d(TAG, "url:" + url);

        remoteSdk.setRemoteSdkListener(new RemoteSdk.RemoteSdkListener() {
            @Override
            public void onSignalConnStatus(RemoteSdk.ConnStatus status, String code, String msg) {
                Log.d(TAG,  "onWebsocketConnStatus: "+ status + " msg: " + msg);
                switch (status) {
                    case CONNECTING:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.VISIBLE);
                        connectLayout.setVisibility(View.GONE);
                        break;
                    case CONNECTED:
                        progressBar.setVisibility(View.GONE);
                        connectLayout.setVisibility(View.GONE);
                        overlay.setVisibility(View.GONE);
                        remoteSdk.connectVideo(); // 连接视频
                        if (SpUtils.inst(getApplicationContext()).getCameraEnable()) {
                            connectCameraAudio();
                        }
                        break;
                    case DISCONNECTED:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.GONE);
                        errMsg.setText("连接断开: " + code + " " + getErrMsg(code));
                        connectLayout.setVisibility(View.VISIBLE);
                        remoteSdk.disconnectVideo();
                        remoteSdk.disconnectCamera();
                        break;
                }
            }

            @Override
            public void onVideoConnStatusChange(RemoteSdk.ConnStatus status, String code, String msg) {
                Log.d(TAG,  "onVideoConnStatusChange: "+ status + " msg: " + msg);
                switch (status) {
                    case CONNECTING:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.VISIBLE);
                        connectLayout.setVisibility(View.GONE);
                        break;
                    case CONNECTED:
                        progressBar.setVisibility(View.GONE);
                        connectLayout.setVisibility(View.GONE);
                        overlay.setVisibility(View.GONE);
                        break;
                    case DISCONNECTED:
                        overlay.setVisibility(View.VISIBLE);
                        progressBar.setVisibility(View.GONE);
                        errMsg.setText("连接断开: " + code + " " + getErrMsg(code));
                        connectLayout.setVisibility(View.VISIBLE);
                        break;
                }
            }

            @Override
            public void onVideoOrientationChange(RemoteSdk.OrientationEnum orientation, int width, int height) {
                if (orientation == RemoteSdk.OrientationEnum.VERTICAL) {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                    bottomBar.setVisibility(View.VISIBLE);
                } else {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                    bottomBar.setVisibility(View.GONE);
                }
            }

            @Override
            public void onCameraAudioConnStatusChange(RemoteSdk.ConnStatus status, String code, String msg) {
                Log.d(TAG, "onCameraAudioConnStatusChange : " + status);
                if (status == RemoteSdk.ConnStatus.CONNECTED) {
                    Toast.makeText(context, "相机启用连接成功", Toast.LENGTH_SHORT).show();
                } else if (status == RemoteSdk.ConnStatus.DISCONNECTED) {
                    Toast.makeText(context, "相机连接断开", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onCopy(String text) {
                Log.d(TAG, "收到剪贴版内容：" + text);
                //Toast.makeText(context, "收到剪贴版内容："  + text, Toast.LENGTH_SHORT).show();
                ClipboardUtils.set(getApplicationContext(), text);
            }
        });
        remoteSdk.setRemoteView(this.remoteView);

        changeResolution(SpUtils.inst(this).getResolution());

        if (!TextUtils.isEmpty(this.url)) {
            remoteSdk.connectSignal(this.url);
        }
    }

    void initBottomMenu() {
        bottomMenuDialog = new BottomMenuDialog(this, new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                bottomMenuDialog.hide();
                switch (v.getId()) {
                    case R.id.btn_clipboard:
                        inputDialog.show("");
                        break;
                    case R.id.btn_clean_app:
                        remoteSdk.cleanApp(false).request(new IResponseCallback() {
                            @Override
                            public void onSuccess(Object data) {
                                Toast.makeText(context, "操作成功", Toast.LENGTH_SHORT).show();
                            }

                            @Override
                            public void onFailed(ResponseCode responseCode) {
                                Toast.makeText(context, "操作失败：" + responseCode.getMsg(), Toast.LENGTH_SHORT).show();
                            }
                        });
                        break;
                    case R.id.btn_shake:
                        remoteSdk.shake().request(new IResponseCallback() {
                            @Override
                            public void onSuccess(Object data) {
                                Toast.makeText(context, "操作成功", Toast.LENGTH_SHORT).show();
                            }

                            @Override
                            public void onFailed(ResponseCode responseCode) {
                                Toast.makeText(context, "操作失败：" + responseCode.getMsg(), Toast.LENGTH_SHORT).show();
                            }
                        });
                        break;
                    case R.id.btn_camera:
                        boolean enable = SpUtils.inst(getApplicationContext()).getCameraEnable();
                        if (enable) {
                            disconnectCameraAudio();
                        } else {
                            connectCameraAudio();
                        }
                        break;
                    case R.id.btn_resolution:
                        resolutionDialog.show();
                        break;
                    case R.id.btn_exit:
                        finish();
                        break;
                }
            }
        });
    }

    void initInputDialog() {
        inputDialog = new InputDialog(this, new InputDialog.OnOkListener() {
            @Override
            public void onOk(String text) {
                remoteSdk.sendPaste(text);
                Toast.makeText(context, "操作成功", Toast.LENGTH_SHORT).show();
                inputDialog.hide();
            }
        });
    }

    void initResolutionDialog() {
        resolutionDialog = new ResolutionDialog(this, new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                resolutionDialog.hide();
                switch (v.getId()) {
                    case R.id.btn_high:
                        changeResolution(RemoteSdk.ResolutionEnum.P720);
                        break;
                    case R.id.btn_middle:
                        changeResolution(RemoteSdk.ResolutionEnum.P480);
                        break;
                    case R.id.btn_low:
                        changeResolution(RemoteSdk.ResolutionEnum.P360);
                        break;
                }
            }
        });
    }

    void changeResolution(RemoteSdk.ResolutionEnum resolutionEnum) {
        remoteSdk.setResolution(resolutionEnum);
        if (resolutionEnum == RemoteSdk.ResolutionEnum.P720 || resolutionEnum == RemoteSdk.ResolutionEnum.P1080) {
            btnResolution.setText("高清");
            bottomMenuDialog.setResolution("高清");
        } else if (resolutionEnum == RemoteSdk.ResolutionEnum.P480) {
            btnResolution.setText("清晰");
            bottomMenuDialog.setResolution("清晰");
        } else {
            btnResolution.setText("流畅");
            bottomMenuDialog.setResolution("流畅");
        }
        SpUtils.inst(context).setResolution(resolutionEnum);
    }

    void setCameraBtn(boolean enable) {
        bottomMenuDialog.updateCameraImg(enable);
    }

    public String getErrMsg(String code) {
        if (RemoteSdk.ErrCode.INVALID_TOKEN.equals(code)) {
            return "会话过期";
        } else if (RemoteSdk.ErrCode.CONNECT_TOO_OFTEN.equals(code)){
            return "连接太频繁，5秒后再试";
        } else if (RemoteSdk.ErrCode.INVALID_TOKEN.equals(code)){
            return "连接太频繁，5秒后再试";
        } else if (RemoteSdk.ErrCode.TIMEOUT.equals(code)){
            return "连接超时";
        } else if (RemoteSdk.ErrCode.VIDEO_DISCONNECTED.equals(code)){
            return "视频流断开";
        } else if (RemoteSdk.ErrCode.SINGNAL_DISCONNECTED.equals(code)){
            return "Websocket断开";
        } else if (RemoteSdk.ErrCode.INVALID_URL.equals(code)){
            return "URL格式不合法";
        }
        return "未知错误";
    }

    public static void start(Context context, String url) {
        Intent starter = new Intent(context, RemoteActivity.class);
        starter.putExtra(KEY_URL, url);
        context.startActivity(starter);
    }

    /** 权限检查
     * @param neededPermissions 需要的权限
     * @return 是否全部被允许
     */
    protected boolean checkPermissions(String[] neededPermissions) {
        if (neededPermissions == null || neededPermissions.length == 0) {
            return true;
        }
        boolean allGranted = true;
        for (String neededPermission : neededPermissions) {
            allGranted &= ContextCompat.checkSelfPermission(this, neededPermission) == PackageManager.PERMISSION_GRANTED;
        }
        return allGranted;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == ACTION_REQUEST_PERMISSIONS) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                connectCameraAudio();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        remoteSdk.sendPaste(ClipboardUtils.get(getApplicationContext()));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (remoteSdk != null) {
            remoteSdk.disconnectSignal();
            remoteSdk.deInit();
        }
        if (remoteView != null) {
            remoteView.release();
        }
        bottomMenuDialog.dismiss();
        inputDialog.dismiss();
        resolutionDialog.dismiss();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnBack:
                remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_BACK);
                break;
            case R.id.btnHome:
                remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_HOME);
                break;
            case R.id.btnTask:
                remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_APP_SWITCH);
                break;
            case R.id.btnVoiceUp:
                remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_VOLUME_UP);
                break;
            case R.id.btnVoiceDown:
                remoteSdk.sendClickKeyEvent(KeyEvent.KEYCODE_VOLUME_DOWN);
                break;
            case R.id.btnResolution:
                resolutionDialog.show();
                break;
            case R.id.btnMore:
                bottomMenuDialog.show();
                break;
        }
    }

    void disconnectCameraAudio() {
        remoteSdk.disconnectCamera();
        SpUtils.inst(getApplicationContext()).setCameraEnable(false);
        setCameraBtn(false);
    }

    void connectCameraAudio() {
        if (!checkPermissions(NEEDED_PERMISSIONS)) {
            ActivityCompat.requestPermissions(this, NEEDED_PERMISSIONS,
                    ACTION_REQUEST_PERMISSIONS);
            return;
        }
        // 默认用哪个摄像头 true 前置摄像头  false 后置摄像头， 实际会受云手机内调用摄像头的APP控制
        remoteSdk.connectCamera(true);
        SpUtils.inst(getApplicationContext()).setCameraEnable(true);
        setCameraBtn(true);
    }

}