package com.chinac.sdk.demo.remote.widget;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ValueAnimator;
import android.app.Service;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.animation.BounceInterpolator;

import com.chinac.sdk.demo.remote.R;

import java.util.ArrayList;
import java.util.List;

/**
 * TODO: document your custom view class.
 */
public class FloatBall extends View {
    final boolean DEBUG = true;
    private final static String TAG = "FloatBall";
    private final static long ANIMATE_DURATION = 300; // 展开动画时间
    Context mContext;

    final int DIRECT_RIGHT = 1;
    final int DIRECT_LEFT = -1;

    Point mPoint = new Point();
    int mDirection = DIRECT_RIGHT; //right
    boolean mMove;
    int mScreenWidth;
    int mScreenHeight;

    float mRadius = 200; // 中心点到展开图标圆心的半径
    float mIconWidth = 100;
    float mIconHeight = mIconWidth;
    final static float scaleBg = 1.4f;
    float mBgCircleRadius = mIconWidth * scaleBg / 2;
    int debounce = 5; // 防抖距离

    //Rect centerRect = new Rect(0, 0, (int) mIconWidth, (int) mIconHeight); // 中心图标绘制区域
    Rect mAvailableRect = new Rect(0, 0, (int) mIconWidth, (int) mIconHeight);  // 中心点可移动区域

    List<ItemRect> iconList = new ArrayList<>();

    ItemRect mTouchItem;
    private ValueAnimator mOenAnimation;
    private ValueAnimator mCloseAnimator;
    private float mAnimateValue = 1f;
    private OpenState mOpenState = OpenState.CLOSE;
    private Paint mRoundPaint;

    private OnItemClickListener onItemClickListener;
    private OnItemLongClickListener onItemLongClickListener;

    public interface OnItemClickListener {
        void onItemClick(int index, int resId);
    }

    public interface OnItemLongClickListener {
        void onItemLongClick(int index, int resId);
    }

    enum OpenState{
        OPEN,
        OPENING,
        CLOSING,
        CLOSE;
    }

    class ItemRect {
        int index;
        int resId;
        float deltaX;
        float deltaY;
        Drawable drawable;
        Rect rect;
    }

    void log(String str) {
        if (DEBUG) Log.d(TAG, str);
    }
    
    public FloatBall(Context context) {
        super(context);
        init(context,null, 0);
    }

    public FloatBall(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context,attrs, 0);
    }

    public FloatBall(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        init(context,attrs, defStyle);
    }

    void initAvailableRect() {
        Point point =  new Point();
        getDisplay().getRealSize(point);
        mScreenWidth = point.x;
        mScreenHeight = point.y;
        mAvailableRect = new Rect();
        int padding = (int)(mIconWidth / 2.0f);
        mAvailableRect.left = padding;
        mAvailableRect.top = padding;
        mAvailableRect.right = mScreenWidth - padding;
        mAvailableRect.bottom = mScreenHeight - padding;
    }

    private void init(Context context,AttributeSet attrs, int defStyle) {
        mContext = context;
        // Load attributes
        final TypedArray a = getContext().obtainStyledAttributes(
                attrs, R.styleable.FloatBall, defStyle, 0);
        // Use getDimensionPixelSize or getDimensionPixelOffset when dealing with
        // values that should fall on pixel boundaries.
        mRadius = a.getDimension(
                R.styleable.FloatBall_floatBallRadius,
                mRadius);
        mIconWidth = mIconHeight =  a.getDimension(
                R.styleable.FloatBall_floatBallSize,
                mIconWidth);
        mBgCircleRadius = mIconWidth * scaleBg / 2f;
        int iconsResId = a.getResourceId(R.styleable.FloatBall_floatBallIcons, R.array.float_icons);
        TypedArray iconsArr = getResources().obtainTypedArray(iconsResId);
        setIconList(iconsArr);
        a.recycle();
        mRoundPaint = new Paint();
        mRoundPaint.setColor(Color.parseColor("#80000000"));
        mRoundPaint.setStyle(Paint.Style.FILL);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);

        int width = getMeasuredWidth();
        int height = getMeasuredHeight();
        log("onMeasure width:" + width + " height:" + height);
        mAvailableRect.left = (int) mBgCircleRadius;
        mAvailableRect.top = (int) (mBgCircleRadius + mRadius);
        mAvailableRect.right = (int) (width - mBgCircleRadius);
        mAvailableRect.bottom = (int)(height - mBgCircleRadius - mRadius);
        if (mPoint.y == 0) { // 初始
            mPoint.y = height / 2;
        }
        if (mDirection == DIRECT_LEFT) {
            mPoint.x = 0;
        } else {
            mPoint.x = width;
        }
        calcMovePos(mPoint.x, mPoint.y);
    }

    //    public void setIconList(int drawableIds[]){
//        int n = drawableIds.length;
//        float angle = 1f / (n - 2); // 减去圆心的图，和直径上的图
//        for(int i = 0; i < n; i++) {
//            DrawableRect item = new DrawableRect();
//            item.index = i;
//            item.resId = drawableIds[i];
//            if (i == 0) { // 圆心
//                item.deltaX = 0;
//                item.deltaY = 0;
//            } else if (i == 1) {
//                item.deltaX = 0;
//                item.deltaY = 1 * radius;
//            } else if (i == n -1) {
//                item.deltaX = 0;
//                item.deltaY = -1 * radius;
//            } else {
//                double a = Math.PI * (angle * (i - 1) + 1/2f);
//                item.deltaX = (float)Math.cos(a) * radius;
//                item.deltaY = (float) Math.sin(a) * radius;
//            }
//
//            item.drawable = getResources().getDrawable(drawableIds[i], null);
//            item.rect = new Rect();
//            iconList.add(item);
//        }
//    }

    public void setIconList(TypedArray typedArray){
        int n = typedArray.length();
        float angle = 1f / (n - 2); // 减去圆心的图，和直径上的图
        for(int i = 0; i < n; i++) {
            ItemRect item = new ItemRect();
            item.index = i;
            item.resId = typedArray.getResourceId(i, R.drawable.btn_float_ball);
            log("resId " + item.resId);
            if (i == 0) { // 圆心
                item.deltaX = 0;
                item.deltaY = 0;
            } else if (i == 1) { // 顶上
                item.deltaX = 0;
                item.deltaY = 1 * mRadius;
            } else if (i == n - 1) { // 底下
                item.deltaX = 0;
                item.deltaY = -1 * mRadius;
            } else {
                double a = Math.PI * (angle * (i - 1) + 1/2f);
                item.deltaX = (float)Math.cos(a) * mRadius;
                item.deltaY = (float) Math.sin(a) * mRadius;
            }

            item.drawable = typedArray.getDrawable(i);
            item.rect = new Rect();
            iconList.add(item);
        }
    }

//    private void initIconList(int n) {
//        float angle = 1f / (n - 2); // 减去圆心的图，和直径上的图
//        for(int i = 0; i < n; i++) {
//            DrawableRect item = new DrawableRect();
//            item.index = i;
//            if (i == 0) { // 圆心
//                item.deltaX = 0;
//                item.deltaY = 0;
//            } else if (i == 1) {
//                item.deltaX = 0;
//                item.deltaY = 1 * radius;
//            } else if (i == n -1) {
//                item.deltaX = 0;
//                item.deltaY = -1 * radius;
//            } else {
//                double a = Math.PI * (angle * (i - 1) + 1/2f);
//                item.deltaX = (float)Math.cos(a) * radius;
//                item.deltaY = (float)Math.sin(a) * radius;
//            }
//
//            item.drawable = mfloatBallDrawable;
//            item.rect = new Rect();
//            iconList.add(item);
//        }
//    }


    /**
     * @param canvas
     * @param drawable
     * @param pointX
     * @param pointY
     */
    void drawIcon(Canvas canvas, ItemRect drawable, float pointX, float pointY) {
        Rect centerRect = drawable.rect;
        float x = pointX - mIconWidth / 2;
        float y = pointY - mIconHeight / 2;
        centerRect.left = (int)x;
        centerRect.top = (int)y;
        centerRect.right = (int)(x + mIconWidth);
        centerRect.bottom = (int)(y + mIconHeight);

        canvas.drawCircle(pointX, pointY, mBgCircleRadius, mRoundPaint); // 添加半透明圆形背景
        drawable.drawable.setBounds(centerRect.left, centerRect.top,
                centerRect.right, centerRect.bottom);
        drawable.drawable.draw(canvas);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        log("onDraw " + mAnimateValue + " openState " + mOpenState);
        // TODO: consider storing these as member variables to reduce
        // allocations per draw cycle.

        if (iconList.size() > 0) {
            ItemRect item = iconList.get(0);
            drawIcon(canvas, item, mPoint.x, mPoint.y);

            if (mOpenState != OpenState.CLOSE) {
                for (int i = 1; i < iconList.size(); i++) {
                    item = iconList.get(i);
                    float x = mPoint.x + item.deltaX * mAnimateValue * mDirection;
                    float y = mPoint.y - item.deltaY * mAnimateValue;
                    drawIcon(canvas, item, x, y);
                }
            }
        }
    }

//
    boolean mPress = true;
    long mDownTime = 0;
    int mDownX = 0;
    int mDownY = 0;
    long mUpTime = 0;

    Runnable longClick = new Runnable() {
        @Override
        public void run() {
            if (mTouchItem != null) {
                performLongClickItem(mTouchItem);
                log("======longclick============");
            }
        }
    };

    void performLongClickItem(ItemRect item) {
        //震动
        Vibrator vib = (Vibrator) mContext.getSystemService(Service.VIBRATOR_SERVICE);   //获取系统震动服务
        vib.vibrate(100);
        // 展开不可以移动
        if (mTouchItem.index == 0 && mOpenState != OpenState.OPEN) {
            mMove = true;
        }
        if (this.onItemLongClickListener != null) {
            this.onItemLongClickListener.onItemLongClick(item.index, item.resId);
        }
    }

    void checkLongClick() {
        log("checkLongCLick");
        postDelayed(longClick, ViewConfiguration.getLongPressTimeout());
    }

    void cancelLongClick() {
        removeCallbacks(longClick);
    }

    ItemRect getClickItem(int x, int y) {
        for (int i = 0; i < iconList.size(); i++) {
            ItemRect itemRect = iconList.get(i);
            log("rect:" + itemRect.rect + " x "+ x + " y: " + y);
            if (itemRect.rect.contains(x,y)){
                return itemRect;
            }
        }
        return null;
    }

    // 防抖
    boolean debounce(int x, int y) {
        return Math.sqrt(Math.pow(mDownX - x, 2) + Math.pow(mDownY - y, 2)) > debounce;
    }

    Point calcMovePos(int x, int y) {
        if (x < mAvailableRect.left) {
            mPoint.x = mAvailableRect.left;
        } else if (x > mAvailableRect.right) {
            mPoint.x = mAvailableRect.right;
        } else {
            mPoint.x = x;
        }

        if (y < mAvailableRect.top) {
            mPoint.y = mAvailableRect.top;
        } else if (y > mAvailableRect.bottom) {
            mPoint.y = mAvailableRect.bottom;
        } else {
            mPoint.y = y;
        }
        mDirection = mPoint.x > getMeasuredWidth() / 2 ? DIRECT_RIGHT : DIRECT_LEFT; // 控制左边展开还是右展开
        return mPoint;
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        int action = event.getAction() & MotionEvent.ACTION_MASK;
        int x = (int) event.getX();
        int y = (int) event.getY();
        log("touch:" + mMove + " x:" + x + " y:" + y);
        if (action == MotionEvent.ACTION_DOWN) {
            ItemRect itemRect = getClickItem(x, y);
            if (itemRect != null) {
                mTouchItem = itemRect;
                mDownTime = System.currentTimeMillis();
                mPress = true;
                mDownX = x;
                mDownY = y;
                checkLongClick();
                return true;
            }

        } else if (action == MotionEvent.ACTION_MOVE) {
            // 移动一定距离,
            if (debounce(x, y)) {
                mPress = false;
                mDownX = 0;
                mDownY = 0;
                mTouchItem = null;
                cancelLongClick();
            }
            if (mMove) {
                calcMovePos(x, y);
               // requestLayout();
                invalidate();
                return true;
            }
        } else if (action == MotionEvent.ACTION_UP
                || action == MotionEvent.ACTION_CANCEL
                || action == MotionEvent.ACTION_OUTSIDE) {
            mUpTime = System.currentTimeMillis();
            checkClick();
            cancelLongClick();
            mPress = false;
            mDownX = 0;
            mDownY = 0;
            mTouchItem = null;
            if (mMove) {
                int measureWidth = getMeasuredWidth();
                x = x > measureWidth / 2 ? measureWidth : 0; //抬起贴边
                calcMovePos(x, y);
                invalidate();
                mMove = false;
                return true;
            }
        }
        return super.onTouchEvent(event);
    }

    void checkClick() {
        if (mUpTime - mDownTime < 100) {
            log("click");
            performClickItem(mTouchItem);
        }
    }

    void performClickItem(ItemRect mTouchItem) {
        if (mTouchItem == null) {
            log("mTouchItem == null");
            return;
        }
        log("click" + mTouchItem.index);
        if (mTouchItem.index == 0) {
            if (mOpenState == OpenState.CLOSE) {
                startOpenAnimation();
            } else if (mOpenState == OpenState.OPEN) {
                startCloseAnimation();
            }
        }
        if (this.onItemClickListener != null) {
            this.onItemClickListener.onItemClick(mTouchItem.index, mTouchItem.resId);
        }
    }

    void startOpenAnimation() {
        if (mOpenState != OpenState.CLOSE) {
            return;
        }
        mOpenState = OpenState.OPENING;
        if (mOenAnimation == null) {
            mOenAnimation = new ValueAnimator();
        }
        mOenAnimation.setFloatValues(0.0f, 0.8f, 1.0f);
        mOenAnimation.setDuration(ANIMATE_DURATION);
        mOenAnimation.setInterpolator(new BounceInterpolator());
        mOenAnimation.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                mAnimateValue = (float)animation.getAnimatedValue();
                log( "onAnimationUpdate " + animation.isRunning());
                invalidate();
            }
        });
        mOenAnimation.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                mOpenState = OpenState.OPEN;
                super.onAnimationEnd(animation);
            }
        });
        mOenAnimation.start();
    }

    public void open() {
        if (mOpenState == OpenState.CLOSE) {
            startOpenAnimation();
        }
    }

    public void close() {
        if (mOpenState == OpenState.OPEN) {
            startCloseAnimation();
        }
    }

    void startCloseAnimation() {
        if (mOpenState != OpenState.OPEN) {
            return;
        }
        mOpenState = OpenState.CLOSING;
        mCloseAnimator = new ValueAnimator();
        mCloseAnimator.setFloatValues(1.0f,0.0f);
        mCloseAnimator.setDuration(ANIMATE_DURATION);
        mCloseAnimator.setInterpolator(new AccelerateDecelerateInterpolator());
        mCloseAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                mAnimateValue = (float)animation.getAnimatedValue();
                invalidate();
            }
        });
        mCloseAnimator.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                mOpenState = OpenState.CLOSE;
                super.onAnimationEnd(animation);
            }
        });
        mCloseAnimator.start();
    }

    public void setOnItemClickListener(OnItemClickListener listener) {
        this.onItemClickListener = listener;
    }

    public void setOnItemLongClickListener(OnItemLongClickListener listener) {
        this.onItemLongClickListener = listener;
    }

}